from django.forms import ModelForm, PasswordInput
from des.models import DynamicEmailConfiguration


class DynamicEmailConfigurationForm(ModelForm):
    class Meta:
        model = DynamicEmailConfiguration
        exclude = []
        widgets = {
            "password": PasswordInput(render_value=True),
        }



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-12-10 17:03
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DynamicEmailConfiguration',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('host', models.CharField(blank=True, max_length=256, null=True, verbose_name='Email Host')),
                ('port', models.SmallIntegerField(blank=True, null=True, verbose_name='Email Port')),
                ('from_email', models.CharField(blank=True, max_length=256, null=True, verbose_name='Default From Email')),
                ('username', models.CharField(blank=True, max_length=256, null=True, verbose_name='Email Authentication Username')),
                ('password', models.CharField(blank=True, max_length=256, null=True, verbose_name='Email Authentication Password')),
                ('use_tls', models.BooleanField(default=False, verbose_name='Use TLS')),
                ('use_ssl', models.BooleanField(default=False, verbose_name='Use SSL')),
                ('fail_silently', models.BooleanField(default=False, verbose_name='Fail Silently')),
                ('timeout', models.SmallIntegerField(blank=True, null=True, verbose_name='Email Send Timeout (seconds)')),
            ],
            options={
                'verbose_name': 'Email Configuration',
            },
        ),
    ]



from django.http import HttpResponseRedirect, HttpResponseNotFound
from django.views.decorators.http import require_http_methods
from django.contrib import messages
from django.template import loader
from django.conf import settings
from django.core.mail import send_mail
from django.utils.translation import ugettext as _
from des.models import DynamicEmailConfiguration
from des.helpers import get_configuration_admin_url

subject = getattr(settings, 'DES_TEST_SUBJECT', _("Test Email"))
text_template = getattr(settings, 'DES_TEST_TEXT_TEMPLATE', "des/test_email.txt")
html_template = getattr(settings, 'DES_TEST_HTML_TEMPLATE', None)

message_text = loader.render_to_string(text_template)
message_html = loader.render_to_string(html_template) if html_template else None


@require_http_methods(["POST"])
def send_test_email(request):

    if request.user is None or not request.user.is_staff:
        return HttpResponseNotFound()

    email = request.POST.get('email', None)
    config = DynamicEmailConfiguration.get_solo()

    if email:
        try:
            send_mail(
                subject,
                message_text,
                config.from_email or None,
                [email],
                html_message = message_html)

            messages.success(request,
                 _("Test email sent. Please check \"{}\" for a "
                 "message with the subject \"{}\"").format(
                    email,
                    subject
                )
            )
        except Exception as e:
            messages.error(request, _("Could not send email. {}").format(e))
    else:
        messages.error(request, _("You must provide an email address to test with."))

    return HttpResponseRedirect(get_configuration_admin_url())


__all__ = ['send_test_email']



# -*- coding: utf-8 -*-
from django.db import models
from django.utils.translation import ugettext as _
from solo.models import SingletonModel
from django.core.exceptions import ValidationError


class DynamicEmailConfiguration(SingletonModel):
    host = models.CharField(
        blank = True, null = True,
        max_length = 256, verbose_name = _("Email Host"))

    port = models.SmallIntegerField(
        blank = True, null = True,
        verbose_name = _("Email Port"))

    from_email = models.CharField(
        blank = True, null = True,
        max_length = 256, verbose_name = _("Default From Email"))

    username = models.CharField(
        blank = True, null = True,
        max_length = 256, verbose_name = _("Email Authentication Username"))

    password = models.CharField(
        blank = True, null = True,
        max_length = 256, verbose_name = _("Email Authentication Password"))

    use_tls = models.BooleanField(
        default = False, verbose_name = _("Use TLS"))

    use_ssl = models.BooleanField(
        default = False, verbose_name = _("Use SSL"))

    fail_silently = models.BooleanField(
        default = False, verbose_name = _("Fail Silently"))

    timeout = models.SmallIntegerField(
        blank = True, null = True,
        verbose_name = _("Email Send Timeout (seconds)"))

    def clean(self):
        if self.use_ssl and self.use_tls:
            raise ValidationError(
                _("\"Use TLS\" and \"Use SSL\" are mutually exclusive, "
                "so only set one of those settings to True."))

    def __str__(self):
        return _("Email Configuration")

    class Meta:
        verbose_name = _("Email Configuration")


__all__ = ['DynamicEmailConfiguration']



## -*- coding: utf-8 -*-
from des.models import DynamicEmailConfiguration
from django.contrib import admin
from solo.admin import SingletonModelAdmin
from des.forms import DynamicEmailConfigurationForm


class DynamicEmailConfigurationAdmin(SingletonModelAdmin):
    form = DynamicEmailConfigurationForm
    change_form_template = 'des/change_form.html'
    class Media:
        js = ('js/des.js'),
        css = {
            'all': ('css/des.css',)
        }

admin.site.register(DynamicEmailConfiguration, DynamicEmailConfigurationAdmin)



from des.models import DynamicEmailConfiguration
try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse


def get_configuration_admin_url():
    meta = DynamicEmailConfiguration._meta
    return reverse('admin:{}_{}_change'.format(
        meta.app_label, meta.model_name
    ))

__all__ = ['get_configuration_admin_url']



# -*- coding: utf-8 -*-
from django.conf.urls import url
from des import views


urlpatterns = [
    url(r'^send-test-email', views.send_test_email, name = 'des-test-email'),
]

__all__ = ['urlpatterns']



# -*- coding: utf-8 -*-
from django.core.mail.backends.smtp import EmailBackend


class ConfiguredEmailBackend(EmailBackend):
    def __init__(self, host=None, port=None, username=None, password=None,
                 use_tls=None, fail_silently=None, use_ssl=None, timeout=None,
                 ssl_keyfile=None, ssl_certfile=None,
                 **kwargs):

        from des.models import DynamicEmailConfiguration
        configuration = DynamicEmailConfiguration.get_solo()

        super(ConfiguredEmailBackend, self).__init__(
             host = configuration.host if host is None else host,
             port = configuration.port if port is None else port,
             username = configuration.username if username is None else username,
             password = configuration.password if password is None else password,
             use_tls = configuration.use_tls if use_tls is None else use_tls,
             fail_silently = configuration.fail_silently if fail_silently is None else fail_silently,
             use_ssl = configuration.use_ssl if use_ssl is None else use_ssl,
             timeout = configuration.timeout if timeout is None else timeout,
             ssl_keyfile = ssl_keyfile, # TODO: configuration.ssl_keyfile if ssl_keyfile is not None else ssl_keyfile,
             ssl_certfile = ssl_certfile, # TODO: configuration.ssl_certfile if ssl_certfile is not None else ssl_certfile,
             **kwargs)


__all__ = ['ConfiguredEmailBackend']
